<?php
/*
 * Plugin Name: Store Manager Connector
 * Plugin URI: https://emagicone.com/products/store-manager-for-woocommerce/
 * Description: Store Manager Connector is a plugin, intended to connect Store Manager software to your WooCommerce store.
 * Author: eMagicOne
 * Author URI: https://emagicone.com/
 * Version: 1.1.2
 * Text Domain: store-manager-connector
 * Domain Path: /i18n
 * License: GPL version 2 or later - http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
*/

/*-----------------------------------------------------------------------------+
| eMagicOne                                                                    |
| Copyright (c) 2022 eMagicOne.com <contact@emagicone.com>		               |
| All rights reserved                                                          |
+------------------------------------------------------------------------------+
|                                                                              |
| Store Manager Connector					                                   |
|                                                                              |
| Developed by eMagicOne,                                                      |
| Copyright (c) 2022                                            	           |
+-----------------------------------------------------------------------------*/

if (! defined('ABSPATH')) {
    exit;
}

define('EMO_SMC_CRYPT_KEY', "EMO_smconnector\0\0\0\0\0\0\0\0\0");
define('EMO_SMC_DEFAULT_LOGIN', '1');
define('EMO_SMC_DEFAULT_PASSWORD', '1');
//define('EMO_SMC_DEFAULT_TMP_DIR', '/wp-content/plugins/store-manager-connector/tmp');
define('EMO_SMC_DEFAULT_ALLOW_COMPRESSION', 1);
define('EMO_SMC_DEFAULT_USE_PLUGIN_TMP_DIR', 0);
define('EMO_SMC_DEFAULT_COMPRESS_LEVEL', 6);        // 1 - 9
define('EMO_SMC_DEFAULT_LIMIT_QUERY_SIZE', 8192);   // kB
define('EMO_SMC_DEFAULT_PACKAGE_SIZE', 1024);       // kB
define('EMO_SMC_DEFAULT_NOTIFICATION_EMAIL', '');
define('EMO_SMC_DEFAULT_ALLOWED_IPS', '');
define('EMO_SMC_MIN_COMPRESS_LEVEL', 1);
define('EMO_SMC_MAX_COMPRESS_LEVEL', 9);
define('EMO_SMC_MIN_LIMIT_QUERY_SIZE', 100);    // kB
define('EMO_SMC_MAX_LIMIT_QUERY_SIZE', 100000); // kB
define('EMO_SMC_MIN_PACKAGE_SIZE', 100);        // kB
define('EMO_SMC_MAX_PACKAGE_SIZE', 30000);      // kB

function emo_smc_get_tmp_path()
{
    $emo_plugin_path = plugin_dir_path(__FILE__);
    if (strpos($emo_plugin_path, ABSPATH) !== false) {
        $emo_tmp_dir = substr($emo_plugin_path, strlen(ABSPATH) - 1) . 'tmp';
    } else {
        $emo_tmp_dir = ABSPATH . 'tmp';
    }

    return $emo_tmp_dir;
}

function emo_smc_get_plugin_tmp_path()
{
    return plugin_dir_path(__FILE__) . 'tmp';
}

function emo_smc_get_encrypted_password($data)
{
    return call_user_func(
        'base64_encode',
        openssl_encrypt(
            $data,
            'aes-192-ecb',
            EMO_SMC_CRYPT_KEY,
            OPENSSL_RAW_DATA
        )
    );
}

function emo_smc_get_decrypted_password($data)
{
    return trim(
        preg_replace(
            "/(^\s+)|(\s+$)/us",
            '',
            openssl_decrypt(
                call_user_func(
                    'base64_decode',
                    $data
                ),
                'aes-192-ecb',
                EMO_SMC_CRYPT_KEY,
                OPENSSL_RAW_DATA | OPENSSL_ZERO_PADDING
            )
        ),
        "\x00..\x1F"
    );
}

function emo_smc_get_default_excluded_tables()
{
    $table_excluded = array();
    return $table_excluded;
}

function emo_smc_get_default_smconnector_options()
{
    $option_values = array(
        'login'              => EMO_SMC_DEFAULT_LOGIN,
        'password'           => emo_smc_get_encrypted_password(EMO_SMC_DEFAULT_PASSWORD),
        'smconnector_hash'   => md5(EMO_SMC_DEFAULT_LOGIN . EMO_SMC_DEFAULT_PASSWORD),
        'tmp_dir'            => emo_smc_get_tmp_path(),
        'use_plugin_tmp_dir' => EMO_SMC_DEFAULT_USE_PLUGIN_TMP_DIR,
        'allow_compression'  => EMO_SMC_DEFAULT_ALLOW_COMPRESSION,
        'compress_level'     => EMO_SMC_DEFAULT_COMPRESS_LEVEL,
        'limit_query_size'   => EMO_SMC_DEFAULT_LIMIT_QUERY_SIZE,
        'package_size'       => EMO_SMC_DEFAULT_PACKAGE_SIZE,
        'exclude_db_tables'  => implode(';', emo_smc_get_default_excluded_tables()),
        'notification_email' => EMO_SMC_DEFAULT_NOTIFICATION_EMAIL,
        'allowed_ips'        => EMO_SMC_DEFAULT_ALLOWED_IPS,
    );

    return $option_values;
}

function emo_smc_check_is_woocommerce_activated()
{
    include_once str_replace(site_url() . '/', ABSPATH, admin_url('includes/plugin.php'));

    $plugin_status = false;
    $separator = substr(__DIR__, -24, 1);
    $path = str_replace($separator.'store-manager-connector', '', __DIR__);
    $results = scandir($path, SCANDIR_SORT_ASCENDING);
    foreach ($results as $result) {
        if ($result === '.' || $result === '..') {
            continue;
        }

        $woocommerce_plugin_path = "$path/$result";
        if (is_dir($woocommerce_plugin_path)
            && (bool)preg_match("/^woocommerce((\-\d+)?(\.\d+)?(\.\d+)?)$/", $result) !== false
            && file_exists("$woocommerce_plugin_path/woocommerce.php")
            && (is_plugin_active("$result/woocommerce.php")
                || in_array(
                    "$result/woocommerce.php",
                    apply_filters(
                        'active_plugins',
                        get_option('active_plugins')
                    ),
                    true
                    )
                )
        ) {
            $plugin_status = true;
            break;
        }
    }

    return $plugin_status;
}

/**
 * Check if WooCommerce is active
 */
if (emo_smc_check_is_woocommerce_activated()) {
    if (!class_exists('EmoSMConnectorStart')) {
		class EmoSMConnectorStart
        {
			public function __construct()
            {
				add_action('admin_enqueue_scripts', array(&$this,'registerOptionStylesAndJavascript'));
				add_filter('query_vars', array(&$this,'addQueryVars'));
				add_action('template_redirect', array(&$this,'theTemplate'));
				add_action('plugins_loaded', array(&$this, 'translations'));

				register_activation_hook(__FILE__, array(&$this, 'smconnectorActivation'));
				register_deactivation_hook(__FILE__, array(&$this, 'smconnectorDeactivation'));

				$plugin = plugin_basename(__FILE__);
				add_filter("plugin_action_links_$plugin", array( &$this,'settingLink'));

			}

			private function createTables()
			{
				global $wpdb;
                require_once str_replace(site_url() .'/', ABSPATH, admin_url('includes/upgrade.php'));

				// Create table `smconnector_session_keys`
                dbDelta(
					"CREATE TABLE IF NOT EXISTS `smconnector_session_keys` (
					`id` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
					`session_key` VARCHAR(100) NOT NULL,
					`date_added` DATETIME NOT NULL,
					`last_activity` DATETIME NOT NULL,
					PRIMARY KEY (`id`))"
				);

				// Create table `smconnector_failed_login`
                dbDelta(
					"CREATE TABLE IF NOT EXISTS `smconnector_failed_login` (
					`id` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
					`ip` VARCHAR(20) NOT NULL,
					`date_added` DATETIME NOT NULL,
					PRIMARY KEY (`id`))"
				);
			}
			
			public function addQueryVars($vars)
            {
				$vars[] = 'connector';
				$vars[] = 'task';
				$vars[] = 'category';
				$vars[] = 'include_tables';
				$vars[] = 'sql';
				$vars[] = 'filename';
				$vars[] = 'position';
				$vars[] = 'search_path';
				$vars[] = 'include_subdir';
				$vars[] = 'mask';
				$vars[] = 'ignore_dir';
				$vars[] = 'checksum_sm';
				$vars[] = 'checksum_sm';
				$vars[] = 'hash';
				$vars[] = 'entity_type';
				$vars[] = 'image_id';
				$vars[] = 'key';

				return $vars;
			}

			public function theTemplate($template)
            {
				global $wp_query;

				if (!isset($wp_query->query['connector'])) {
					return $template;
				}

				if ($wp_query->query['connector'] == 'bridge') {
                    new EmoStoreManagerConnector();
					exit;
				}

				return $template;
			}

			public function smconnectorDeactivation()
            {
                if (!current_user_can('activate_plugins'))
                    return;

                $plugin = isset($_REQUEST['plugin']) ? sanitize_text_field($_REQUEST['plugin']) : '';
                check_admin_referer("deactivate-plugin_{$plugin}");
			}

			public function smconnectorActivation()
            {
				global $wpdb;

                if (!current_user_can('activate_plugins')) {
					return;
				}

                $plugin = isset($_REQUEST['plugin']) ? sanitize_text_field($_REQUEST['plugin']) : '';
                check_admin_referer("activate-plugin_{$plugin}");

                if (!(get_option('SMCONNECTOR_OPTIONS'))) {
					$wpdb->replace(
						$wpdb->options,
						array(
							'option_name' => 'SMCONNECTOR_OPTIONS',
							'option_value' => serialize(emo_smc_get_default_smconnector_options())
						)
					);
				}

				$this->createTables();
            }

			public function translations()
            {
                load_plugin_textdomain('store-manager-connector', false, basename(__DIR__) . '/lang');
			}

			public function registerOptionStylesAndJavascript()
            {
                global $hook_suffix;

                if ($hook_suffix === "toplevel_page_smconnector") {
                    wp_register_style('ema_style', plugins_url('assets/css/style.css', __FILE__));
                    wp_enqueue_style('ema_style');
        
                    wp_register_style('tb_style', plugins_url('assets/css/tb.css', __FILE__));
                    wp_enqueue_style('tb_style');

                    wp_register_script('ema_script', plugins_url('assets/js/main.js', __FILE__));
                    wp_enqueue_script('ema_script');
                }
			}

			// Add settings link on plugin page
			public function settingLink($links)
            {
				$settings_link = '<a href="options-general.php?page=smconnector">Settings</a>';
				array_unshift($links, $settings_link);

				return $links;
			}
		}

        $GLOBALS['EmoSMConnectorStart'] = new EmoSMConnectorStart();
        include_once plugin_dir_path(__FILE__) . 'option.php';
        include_once plugin_dir_path(__FILE__) . 'woo-overrider.php';
	}
} else {
	add_action('admin_notices', 'emo_smc_admin_notices');
}

function emo_smc_admin_notices()
{
    echo '<div id="notice" class="error"><p>';
    echo '<b> Store Manager Connector </b> add-on requires <a href="https://woocommerce.com/"> WooCommerce </a> plugin. Please install and activate it.';
    echo '</p></div>', "\n";
}